@@ -1,6 +1,6 @@
 pub mod mesh_data;
-pub mod skel_data;
 pub mod modl_data;
+pub mod skel_data;
 
 use std::io::{Read, Write};
 use std::ops::Mul;
@@ -9,6 +9,7 @@ use binread::io::{Seek, SeekFrom};
 use binread::BinReaderExt;
 use binread::{BinRead, BinResult};
 use half::f16;
+use ssbh_lib::SsbhArray;
 
 fn read_data<R: Read + Seek, TIn: BinRead, TOut: From<TIn>>(
     reader: &mut R,
@@ -68,7 +69,11 @@ fn write_f16<W: Write>(writer: &mut W, data: &[f32]) -> std::io::Result<()> {
     Ok(())
 }
 
-fn write_vector_data<W: Write + Seek, F: Fn(&mut W, &[f32]) -> std::io::Result<()>, const N: usize>(
+fn write_vector_data<
+    W: Write + Seek,
+    F: Fn(&mut W, &[f32]) -> std::io::Result<()>,
+    const N: usize,
+>(
     writer: &mut W,
     elements: &[[f32; N]],
     offset: u64,
@@ -82,6 +87,11 @@ fn write_vector_data<W: Write + Seek, F: Fn(&mut W, &[f32]) -> std::io::Result<(
     Ok(())
 }
 
+// TODO: Should this be part of SsbhLib?
+fn create_ssbh_array<T, B: BinRead, F: Fn(&T) -> B>(elements: &[T], create_b: F) -> SsbhArray<B> {
+    elements.iter().map(create_b).collect::<Vec<B>>().into()
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -206,7 +216,7 @@ mod tests {
     fn u8_clamped() {
         assert_eq!(0u8, get_u8_clamped(-1.0f32));
         assert_eq!(0u8, get_u8_clamped(0.0f32));
-        assert_eq!(128u8, get_u8_clamped(128u8 as f32 / 255f32));
+        assert_eq!(128u8, get_u8_clamped(128f32 / 255f32));
         assert_eq!(255u8, get_u8_clamped(1.0f32));
         assert_eq!(255u8, get_u8_clamped(2.0f32));
     }
