@@ -70,16 +70,16 @@ impl SkelData {
 ```rust
 # use ssbh_data::skel_data::calculate_relative_transform;
 let world_transform = [
-    [2f32, 0f32, 0f32, 0f32],
-    [0f32, 4f32, 0f32, 0f32],
-    [0f32, 0f32, 8f32, 0f32],
-    [1f32, 2f32, 3f32, 1f32],
+    [2.0, 0.0, 0.0, 0.0],
+    [0.0, 4.0, 0.0, 0.0],
+    [0.0, 0.0, 8.0, 0.0],
+    [1.0, 2.0, 3.0, 1.0],
 ];
 let parent_world_transform = [
-    [1f32, 0f32, 0f32, 0f32],
-    [0f32, 1f32, 0f32, 0f32],
-    [0f32, 0f32, 1f32, 0f32],
-    [0f32, 0f32, 0f32, 1f32],
+    [1.0, 0.0, 0.0, 0.0],
+    [0.0, 1.0, 0.0, 0.0],
+    [0.0, 0.0, 1.0, 0.0],
+    [0.0, 0.0, 0.0, 1.0],
 ];
 assert_eq!(
     world_transform,
@@ -182,6 +182,7 @@ fn mat4_from_row2d(elements: &[[f32; 4]; 4]) -> Mat4 {
 
 impl SkelData {
     /// Calculates the world transform for `bone` by accumulating the transform with the parents transform recursively.
+    /// For single bound objects, the object is transformed by the parent bone's world transform.
     /// Returns the resulting matrix in row-major order.
     /**
     ```rust
@@ -190,12 +191,15 @@ impl SkelData {
     #     major_version: 1,
     #     minor_version: 0,
     #     bones: vec![BoneData {
-    #         name: "root".to_string(),
+    #         name: "Head".to_string(),
     #         transform: [[0f32; 4]; 4],
     #         parent_index: None,
     #     }],
     # };
-    let world_transform = data.calculate_world_transform(&data.bones[0]);
+    let parent_bone_name = "Head";
+    if let Some(parent_bone) = data.bones.iter().find(|b| b.name == parent_bone_name) {
+        let world_transform = data.calculate_world_transform(&parent_bone);
+    }
     ```
     */
     pub fn calculate_world_transform(&self, bone: &BoneData) -> [[f32; 4]; 4] {
@@ -239,10 +243,10 @@ mod tests {
         assert_eq!("abc", data.name);
         assert_eq!(
             [
-                [1f32, 0f32, 0f32, 0f32],
-                [0f32, 1f32, 0f32, 0f32],
-                [0f32, 0f32, 1f32, 0f32],
-                [0f32, 0f32, 0f32, 1f32]
+                [1.0, 0.0, 0.0, 0.0],
+                [0.0, 1.0, 0.0, 0.0],
+                [0.0, 0.0, 1.0, 0.0],
+                [0.0, 0.0, 0.0, 1.0]
             ],
             data.transform
         );
@@ -270,22 +274,22 @@ mod tests {
     #[test]
     fn calculate_relative_transform_with_parent() {
         let world_transform = [
-            [2f32, 0f32, 0f32, 0f32],
-            [0f32, 4f32, 0f32, 0f32],
-            [0f32, 0f32, 8f32, 0f32],
-            [0f32, 0f32, 0f32, 1f32],
+            [2.0, 0.0, 0.0, 0.0],
+            [0.0, 4.0, 0.0, 0.0],
+            [0.0, 0.0, 8.0, 0.0],
+            [0.0, 0.0, 0.0, 1.0],
         ];
         let parent_world_transform = [
-            [1f32, 0f32, 0f32, 0f32],
-            [0f32, 1f32, 0f32, 0f32],
-            [0f32, 0f32, 1f32, 0f32],
-            [1f32, 2f32, 3f32, 1f32],
+            [1.0, 0.0, 0.0, 0.0],
+            [0.0, 1.0, 0.0, 0.0],
+            [0.0, 0.0, 1.0, 0.0],
+            [1.0, 2.0, 3.0, 1.0],
         ];
         let relative_transform = [
-            [2.0f32, 0f32, 0f32, 0f32],
-            [0f32, 4f32, 0f32, 0f32],
-            [0f32, 0f32, 8f32, 0f32],
-            [-2f32, -8f32, -24f32, 1f32],
+            [2.0, 0.0, 0.0, 0.0],
+            [0.0, 4.0, 0.0, 0.0],
+            [0.0, 0.0, 8.0, 0.0],
+            [-2.0, -8.0, -24.0, 1.0],
         ];
         assert_eq!(
             relative_transform,
@@ -296,10 +300,10 @@ mod tests {
     #[test]
     fn calculate_relative_transform_no_parent() {
         let world_transform = [
-            [0f32, 1f32, 2f32, 3f32],
-            [4f32, 5f32, 6f32, 7f32],
-            [8f32, 9f32, 10f32, 11f32],
-            [12f32, 13f32, 14f32, 15f32],
+            [0.0, 1.0, 2.0, 3.0],
+            [4.0, 5.0, 6.0, 7.0],
+            [8.0, 9.0, 10.0, 11.0],
+            [12.0, 13.0, 14.0, 15.0],
         ];
         assert_eq!(
             world_transform,
@@ -318,10 +322,10 @@ mod tests {
     #[test]
     fn test_matrix_relative_eq() {
         let transform = [
-            [0f32, 1f32, 2f32, 3f32],
-            [4f32, 5f32, 6f32, 7f32],
-            [8f32, 9f32, 10f32, 11f32],
-            [12f32, 13f32, 14f32, 15f32],
+            [0.0, 1.0, 2.0, 3.0],
+            [4.0, 5.0, 6.0, 7.0],
+            [8.0, 9.0, 10.0, 11.0],
+            [12.0, 13.0, 14.0, 15.0],
         ];
         assert!(matrices_are_relative_eq(transform, transform));
     }
@@ -330,10 +334,10 @@ mod tests {
     fn world_transform_no_parent() {
         // Use unique values to make sure the matrix is correct.
         let transform = [
-            [0f32, 1f32, 2f32, 3f32],
-            [4f32, 5f32, 6f32, 7f32],
-            [8f32, 9f32, 10f32, 11f32],
-            [12f32, 13f32, 14f32, 15f32],
+            [0.0, 1.0, 2.0, 3.0],
+            [4.0, 5.0, 6.0, 7.0],
+            [8.0, 9.0, 10.0, 11.0],
+            [12.0, 13.0, 14.0, 15.0],
         ];
 
         let data = SkelData {
